const bunyan = require('bunyan');

let logger, loggerName;

function childLogger(moduleName) {
  return logger.child({
    module: moduleName
  });
}

function getEnv(prefix, key) {
  return process.env[prefix + '_' + key] || process.env[key];
}

module.exports = function(name, serializers, opts) {
  const streams = opts ? opts.streams : null;
  if (process.env.NODE_ENV === 'test') {
    const logger = console;
    logger.debug = console.log;
    logger.fatal = console.error;
    return function () {
      return logger;
    };
  }
  if (!name) {
    if (logger) {
      return childLogger;
    } else {
      throw new Error('Must create logger in app before any common modules are loaded and logger must have name');
    }
  } else if (logger) {
    throw new Error('Logger has already been initialized with name ' + loggerName);
  }
  loggerName = name;
  const optionalStreams = Array.isArray(streams) ? streams.slice() : [];
  const logPath = getEnv(name, 'LOG_DIR');
  if (logPath) {
    optionalStreams.push({
      type: 'rotating-file',
      period: '1d',
      count: 30,
      level: 'info',
      path: logPath + '/' + name + '.json',
    });
  }
  logger = bunyan.createLogger({
    name: name,
    streams: [{
        level: 'debug',
        stream: process.stdout
      },
      ...optionalStreams
    ],
    serializers: bunyan.stdSerializers,
  });
  if (serializers) {
      logger.addSerializers(serializers);
  }

  const consoleLogger = logger.child({
    console: true
  });

  console.log = consoleLogger.debug.bind(consoleLogger);

  return childLogger;
}
